#include <afxwin.h>
#include "extapi.h"

// the one and only CExtAPI instance
CExtAPI theAPI;

CExtAPI::CExtAPI(void) {

	m_hID = NULL;

	// app functions

	m_lpfnGetFuncHandle = NULL;
	m_lpfnReportError = NULL;

	// list function handles

	m_lpfn_list_GetList = NULL;
	m_lpfn_list_GetSize = NULL;
	m_lpfn_list_CreateMember = NULL;
	m_lpfn_list_GetMemberByIndex = NULL;
	m_lpfn_list_GetMemberByName = NULL;
	m_lpfn_list_DeleteMemberByIndex = NULL;
	m_lpfn_list_DeleteMemberByName = NULL;
	m_lpfn_list_DeleteMembers = NULL;

	// map functions

	m_lpfn_map_GetMap = NULL;
	m_lpfn_map_Init = NULL;
	m_lpfn_map_Free = NULL;
	m_lpfn_map_GetMapType = NULL;
	m_lpfn_map_GetPixelSize = NULL;
	m_lpfn_map_GetWrapFlag = NULL;
	m_lpfn_map_GetWidth = NULL;
	m_lpfn_map_GetHeight = NULL;
	m_lpfn_map_GetPixel = NULL;
	m_lpfn_map_SetPixel = NULL;
	m_lpfn_map_GetMinMaxAlt = NULL;
	m_lpfn_map_GetHorizScale = NULL;
	m_lpfn_map_GetMosaicFlag = NULL;
	m_lpfn_map_GetMosaicTileSize = NULL;
	m_lpfn_map_GetMosaicTilesX = NULL;
	m_lpfn_map_GetMosaicTilesY = NULL;

	// format functions

	m_lpfn_format_Create = NULL;
	m_lpfn_format_SetFlags = NULL;
	m_lpfn_format_GetExt = NULL;
	m_lpfn_format_GetOptionList = NULL;
	m_lpfn_format_GetOptionValue = NULL;
	m_lpfn_format_SetOptionValue = NULL;

	// prog box function handles

	m_lpfn_progbox_ShowWnd = NULL;
	m_lpfn_progbox_HideWnd = NULL;
	m_lpfn_progbox_SetTitle = NULL;
	m_lpfn_progbox_SetProgress = NULL;

	// combo box function handles

	m_lpfn_combosel_SetOptions = NULL;
	m_lpfn_combosel_GetOptions = NULL;
	m_lpfn_combosel_SetCurSel = NULL;
	m_lpfn_combosel_GetCurSel = NULL;

	// project function handle defs

	m_lpfn_project_GetHeightfieldRange = NULL;
}

CExtAPI::~CExtAPI(void) {
}

bool CExtAPI::InitAPI(FARPROC pFunc, LPVOID hID) {

	if(!pFunc) {
		AfxMessageBox("CL3DTio_HFFApp::InitAPI error:\r\n - Invalid callback handle");
		return false;
	}

	if(!hID) {
		AfxMessageBox("CL3DTio_HFFApp::InitAPI error:\r\n - Invalid ID handle");
		return false;
	}

	//store thr session ID
	m_hID = hID;

	// store the handle to the function by which we load more function handles
	m_lpfnGetFuncHandle = (LPFNAPI_app_GetFuncHandle)pFunc;

	// debugging
//	CString TempStr;
//	TempStr.Format("CExtAPI::InitAPI report:\r\n - GetFuncHandle is 0x%x", m_lpfnGetFuncHandle);
//	AfxMessageBox(TempStr);

	// start loading functions from L3DT

	//
	// app functions
	//

	m_lpfnReportError = (LPFNAPI_app_ReportError)GetFuncHandle("app.ReportError");

	//
	// list functions
	//

	m_lpfn_list_GetList = (LPFNAPI_list_GetList)GetFuncHandle("list.GetList");
	m_lpfn_list_GetSize = (LPFNAPI_list_GetSize)GetFuncHandle("list.GetSize");
	m_lpfn_list_CreateMember = (LPFNAPI_list_CreateMember)GetFuncHandle("list.CreateMember");
	m_lpfn_list_GetMemberByIndex = (LPFNAPI_list_GetMemberByIndex)GetFuncHandle("list.GetMemberByIndex");
	m_lpfn_list_GetMemberByName = (LPFNAPI_list_GetMemberByName)GetFuncHandle("list.GetMemberByName");
	m_lpfn_list_DeleteMemberByIndex = (LPFNAPI_list_DeleteMemberByIndex)GetFuncHandle("list.DeleteMemberByIndex");
	m_lpfn_list_DeleteMemberByName = (LPFNAPI_list_DeleteMemberByName)GetFuncHandle("list.DeleteMemberByName");
	m_lpfn_list_DeleteMembers = (LPFNAPI_list_DeleteMembers)GetFuncHandle("list.DeleteMembers");

	//
	// map functions
	//

	m_lpfn_map_GetMap = (LPFNAPI_map_GetMap)GetFuncHandle("map.GetMap");
	m_lpfn_map_Init = (LPFNAPI_map_Init)GetFuncHandle("map.Init");
	m_lpfn_map_Free = (LPFNAPI_map_Free)GetFuncHandle("map.Free");
	m_lpfn_map_GetMapType = (LPFNAPI_map_GetPixelSize)GetFuncHandle("map.GetMapType");
	m_lpfn_map_GetPixelSize = (LPFNAPI_map_GetPixelSize)GetFuncHandle("map.GetPixelSize");
	m_lpfn_map_GetWrapFlag = (LPFNAPI_map_GetWrapFlag)GetFuncHandle("map.GetWrapFlag");
	m_lpfn_map_GetWidth = (LPFNAPI_map_GetWidth)GetFuncHandle("map.GetWidth");
	m_lpfn_map_GetHeight = (LPFNAPI_map_GetHeight)GetFuncHandle("map.GetHeight");
	m_lpfn_map_GetPixel = (LPFNAPI_map_GetPixel)GetFuncHandle("map.GetPixel");
	m_lpfn_map_SetPixel = (LPFNAPI_map_SetPixel)GetFuncHandle("map.SetPixel");
	m_lpfn_map_GetMinMaxAlt = (LPFNAPI_map_GetMinMaxAlt)GetFuncHandle("map.GetMinMaxAlt");
	m_lpfn_map_GetHorizScale = (LPFNAPI_map_GetHorizScale)GetFuncHandle("map.GetHorizScale");
	m_lpfn_map_GetMosaicFlag = (LPFNAPI_map_GetMosaicFlag)GetFuncHandle("map.GetMosaicFlag");
	m_lpfn_map_GetMosaicTileSize = (LPFNAPI_map_GetMosaicTileSize)GetFuncHandle("map.GetMosaicTileSize");
	m_lpfn_map_GetMosaicTilesX = (LPFNAPI_map_GetMosaicTilesX)GetFuncHandle("map.GetMosaicTilesX");
	m_lpfn_map_GetMosaicTilesY = (LPFNAPI_map_GetMosaicTilesY)GetFuncHandle("map.GetMosaicTilesY");

	//
	// format functions
	//

	m_lpfn_format_Create = (LPFNAPI_format_Create)GetFuncHandle("format.Create");
	m_lpfn_format_SetFlags = (LPFNAPI_format_SetFlags)GetFuncHandle("format.SetFlags");
	m_lpfn_format_GetExt = (LPFNAPI_format_GetExt)GetFuncHandle("format.GetExt");
	m_lpfn_format_GetOptionList = (LPFNAPI_format_GetOptionList)GetFuncHandle("format.GetOptionList");
	m_lpfn_format_GetOptionValue = (LPFNAPI_format_GetOptionValue)GetFuncHandle("format.GetOptionValue");
	m_lpfn_format_SetOptionValue = (LPFNAPI_format_SetOptionValue)GetFuncHandle("format.SetOptionValue");

	//
	// prog box function handles
	//

	m_lpfn_progbox_ShowWnd = (LPFNAPI_progbox_ShowWnd)GetFuncHandle("progbox.ShowWnd");
	m_lpfn_progbox_HideWnd = (LPFNAPI_progbox_HideWnd)GetFuncHandle("progbox.HideWnd");
	m_lpfn_progbox_SetTitle = (LPFNAPI_progbox_SetTitle)GetFuncHandle("progbox.SetTitle");
	m_lpfn_progbox_SetProgress = (LPFNAPI_progbox_SetProgress)GetFuncHandle("progbox.SetProgress");

	//
	// combo box function handles
	//

	m_lpfn_combosel_SetOptions = (LPFNAPI_combosel_SetOptions)GetFuncHandle("combosel.SetOptions");
	m_lpfn_combosel_GetOptions = (LPFNAPI_combosel_GetOptions)GetFuncHandle("combosel.GetOptions");
	m_lpfn_combosel_SetCurSel = (LPFNAPI_combosel_SetCurSel)GetFuncHandle("combosel.SetCurSel");
	m_lpfn_combosel_GetCurSel = (LPFNAPI_combosel_GetCurSel)GetFuncHandle("combosel.GetCurSel");

	//
	// project function handle defs
	//

	m_lpfn_project_GetHeightfieldRange = (LPFNAPI_project_GetHeightfieldRange)GetFuncHandle("project.GetHeightfieldRange");

	return true;
}

//
// app functions
//

FARPROC CExtAPI::GetFuncHandle(LPCTSTR lpFuncName) {
	if(!m_lpfnGetFuncHandle) {
		ReportError("CExtAPI::GetFuncHandle error - API not initialised");
		return NULL;
	}

	FARPROC hProc = m_lpfnGetFuncHandle(m_hID, lpFuncName);
	if(!hProc) {
		ReportError("CExtAPI::GetFuncHandle error - Function not found");
		return NULL;
	}

	return hProc;
}

void CExtAPI::ReportError(LPCSTR lpErrorMessage) {
	
	if(!m_lpfnReportError) {
		AfxMessageBox(lpErrorMessage, MB_ICONERROR);
	}

	m_lpfnReportError(m_hID, lpErrorMessage);
}

//
// list functions
//
ZLIST CExtAPI::list_GetList(LPCSTR lpListName) {
	if(!m_lpfn_list_GetList) {
		ReportError("CExtAPI::list_GetList error - function not found");
		return NULL;
	}

	return m_lpfn_list_GetList(m_hID, lpListName);
}
long CExtAPI::list_GetSize(ZLIST hList) {
	if(!m_lpfn_list_GetSize) {
		ReportError("CExtAPI::list_GetSize error - function not found");
		return 0;
	}

	return m_lpfn_list_GetSize(m_hID, hList);
}
ZLIST CExtAPI::list_CreateMember(ZLIST hList, long VarID, LPCSTR lpVarName) {
	if(!m_lpfn_list_CreateMember) {
		ReportError("CExtAPI::list_CreateMember error - function not found");
		return NULL;
	}

	return m_lpfn_list_CreateMember(m_hID, hList, VarID, lpVarName);
}
ZLIST CExtAPI::list_GetMemberByIndex(ZLIST hList, long index) {
	if(!m_lpfn_list_GetMemberByIndex) {
		ReportError("CExtAPI::list_GetMemberByIndex error - function not found");
		return NULL;
	}

	return m_lpfn_list_GetMemberByIndex(m_hID, hList, index);
}
ZLIST CExtAPI::list_GetMemberByName(ZLIST hList, LPCSTR lpVarName) {
	if(!m_lpfn_list_GetMemberByName) {
		ReportError("CExtAPI::list_GetMemberByName error - function not found");
		return NULL;
	}

	return m_lpfn_list_GetMemberByName(m_hID, hList, lpVarName);
}
bool CExtAPI::list_DeleteMemberByIndex(ZLIST hList, long index) {
	if(!m_lpfn_list_DeleteMemberByIndex) {
		ReportError("CExtAPI::list_DeleteMemberByIndex error - function not found");
		return NULL;
	}

	return m_lpfn_list_DeleteMemberByIndex(m_hID, hList, index);
}
bool CExtAPI::list_DeleteMemberByName(ZLIST hList, LPCSTR lpVarName) {
	if(!m_lpfn_list_DeleteMemberByName) {
		ReportError("CExtAPI::list_DeleteMemberByName error - function not found");
		return NULL;
	}

	return m_lpfn_list_DeleteMemberByName(m_hID, hList, lpVarName);
}
void CExtAPI::list_DeleteMembers(ZLIST hList) {
	if(!m_lpfn_list_GetList) {
		ReportError("CExtAPI::list_DeleteMembers error - function not found");
		return;
	}

	m_lpfn_list_DeleteMembers(m_hID, hList);
}

//
// map functions
//

ZMAP CExtAPI::map_GetMap(LPCSTR lpMapName) {
	if(!m_lpfn_map_GetMap) {
		ReportError("CExtAPI::map_GetMap error - function not found");
		return NULL;
	}

	return m_lpfn_map_GetMap(m_hID, lpMapName);
}

bool CExtAPI::map_Init(ZMAP hMap, long nx, long ny, long MapTypeID, float HorizScale, bool WrapFlag) {
	if(!m_lpfn_map_Init) {
		ReportError("CExtAPI::map_Init error - function not found");
		return false;
	}

	return m_lpfn_map_Init(m_hID, hMap, nx, ny, MapTypeID, HorizScale, WrapFlag);
}
bool CExtAPI::map_Free(ZMAP hMap) {
	if(!m_lpfn_map_Free) {
		ReportError("CExtAPI::map_Free error - function not found");
		return false;
	}

	return m_lpfn_map_Free(m_hID, hMap);
}
long CExtAPI::map_GetMapType(ZMAP hMap) {
	if(!m_lpfn_map_GetMapType) {
		ReportError("CExtAPI::map_GetMapType error - function not found");
		return 0;
	}

	return m_lpfn_map_GetMapType(m_hID, hMap);
}
long CExtAPI::map_GetPixelSize(ZMAP hMap) {
	if(!m_lpfn_map_GetPixelSize) {
		ReportError("CExtAPI::map_GetPixelSize error - function not found");
		return 0;
	}

	return m_lpfn_map_GetPixelSize(m_hID, hMap);
}
bool CExtAPI::map_GetWrapFlag(ZMAP hMap) {
	if(!m_lpfn_map_GetWrapFlag) {
		ReportError("CExtAPI::map_GetWrapFlag error - function not found");
		return false;
	}

	return m_lpfn_map_GetWrapFlag(m_hID, hMap);
}
long CExtAPI::map_GetWidth(ZMAP hMap) {
	if(!m_lpfn_map_GetWidth) {
		ReportError("CExtAPI::map_GetWidth error - function not found");
		return 0;
	}

	return m_lpfn_map_GetWidth(m_hID, hMap);
}
long CExtAPI::map_GetHeight(ZMAP hMap) {
	if(!m_lpfn_map_GetHeight) {
		ReportError("CExtAPI::map_GetHeight error - function not found");
		return 0;
	}

	return m_lpfn_map_GetHeight(m_hID, hMap);
}
bool CExtAPI::map_GetPixel(ZMAP hMap, long x, long y, void* pValue) {
	if(!m_lpfn_map_GetPixel) {
		ReportError("CExtAPI::map_GetPixel error - function not found");
		return false;
	}

	return m_lpfn_map_GetPixel(m_hID, hMap, x, y, pValue);
}
bool CExtAPI::map_SetPixel(ZMAP hMap, long x, long y, void* pValue) {
	if(!m_lpfn_map_SetPixel) {
		ReportError("CExtAPI::map_SetPixel error - function not found");
		return false;
	}

	return m_lpfn_map_SetPixel(m_hID, hMap, x, y, pValue);
}
bool CExtAPI::map_GetMinMaxAlt(ZMAP hMap, float& minval, float& maxval) {
	if(!m_lpfn_map_GetMinMaxAlt) {
		ReportError("CExtAPI::map_GetMinMaxAlt error - function not found");
		return false;
	}

	return m_lpfn_map_GetMinMaxAlt(m_hID, hMap, minval, maxval);
}
float CExtAPI::map_GetHorizScale(ZMAP hMap) {
	if(!m_lpfn_map_GetHorizScale) {
		ReportError("CExtAPI::map_GetHorizScale error - function not found");
		return false;
	}

	return m_lpfn_map_GetHorizScale(m_hID, hMap);
}
bool CExtAPI::map_GetMosaicFlag(ZMAP hMap) {
	if(!m_lpfn_map_GetMosaicFlag) {
		ReportError("CExtAPI::map_GetMosaicFlag error - function not found");
		return false;
	}

	return m_lpfn_map_GetMosaicFlag(m_hID, hMap);
}
long CExtAPI::map_GetMosaicTileSize(ZMAP hMap) {
	if(!m_lpfn_map_GetMosaicTileSize) {
		ReportError("CExtAPI::map_GetMosaicTileSize error - function not found");
		return 0;
	}

	return m_lpfn_map_GetMosaicTileSize(m_hID, hMap);
}
long CExtAPI::map_GetMosaicTilesX(ZMAP hMap) {
	if(!m_lpfn_map_GetMosaicTilesX) {
		ReportError("CExtAPI::map_GetMosaicTilesX error - function not found");
		return 0;
	}

	return m_lpfn_map_GetMosaicTilesX(m_hID, hMap);
}
long CExtAPI::map_GetMosaicTilesY(ZMAP hMap) {
	if(!m_lpfn_map_GetMosaicTilesY) {
		ReportError("CExtAPI::map_GetMosaicTilesY error - function not found");
		return 0;
	}

	return m_lpfn_map_GetMosaicTilesY(m_hID, hMap);
}

//
// format functions
//

ZFORMAT CExtAPI::format_Create(LPCTSTR lpMapName, LPCSTR lpLongName, LPCSTR lpFileExt, LPCSTR lpNativeProgramName) {
	if(!m_lpfn_format_Create) {
		ReportError("CExtAPI::format_Create error - function not found");
		return NULL;
	}

	return m_lpfn_format_Create(m_hID, lpMapName, lpLongName, lpFileExt, lpNativeProgramName);
}
bool CExtAPI::format_SetFlags(ZFORMAT hFormat, bool Native, bool Mosaic) {
	if(!m_lpfn_format_SetFlags) {
		ReportError("CExtAPI::format_SetFlags error - function not found");
		return false;
	}

	return m_lpfn_format_SetFlags(m_hID, hFormat, Native, Mosaic);
}
LPCSTR CExtAPI::format_GetExt(ZFORMAT hFormat) {
	if(!m_lpfn_format_GetExt) {
		ReportError("CExtAPI::format_GetExt error - function not found");
		return NULL;
	}

	return m_lpfn_format_GetExt(m_hID, hFormat);
}
ZLIST CExtAPI::format_GetOptionList(ZFORMAT hFormat) {
	if(!m_lpfn_format_GetOptionList) {
		ReportError("CExtAPI::format_GetOptionList error - function not found");
		return NULL;
	}

	return m_lpfn_format_GetOptionList(m_hID, hFormat);
}
bool CExtAPI::format_SetOptionValue(ZFORMAT hFormat, LPCSTR lpOptionName, long VarID, void* pValue) {
	if(!m_lpfn_format_SetOptionValue) {
		ReportError("CExtAPI::format_SetOptionValue error - function not found");
		return false;
	}

	return m_lpfn_format_SetOptionValue(m_hID, hFormat, lpOptionName, VarID, pValue);
}
bool CExtAPI::format_GetOptionValue(ZFORMAT hFormat, LPCSTR lpOptionName, long VarID, void* pValue) {
	if(!m_lpfn_format_GetOptionValue) {
		ReportError("CExtAPI::format_GetOptionValue error - function not found");
		return false;
	}

	return m_lpfn_format_GetOptionValue(m_hID, hFormat, lpOptionName, VarID, pValue);
}

//
// prog box functions
//

bool CExtAPI::progbox_ShowWnd(ZVAR hProgWnd) {
	if(!m_lpfn_progbox_ShowWnd) {
		ReportError("CExtAPI::progbox_ShowWnd error - function not found");
		return false;
	}

	return m_lpfn_progbox_ShowWnd(m_hID, hProgWnd);
}
bool CExtAPI::progbox_HideWnd(ZVAR hProgWnd) {
	if(!m_lpfn_progbox_HideWnd) {
		ReportError("CExtAPI::progbox_HideWnd error - function not found");
		return false;
	}

	return m_lpfn_progbox_HideWnd(m_hID, hProgWnd);
}
bool CExtAPI::progbox_SetTitle(ZVAR hProgWnd, LPCSTR lpTitleText) {
	if(!m_lpfn_progbox_SetTitle) {
		ReportError("CExtAPI::progbox_SetTitle error - function not found");
		return false;
	}

	return m_lpfn_progbox_SetTitle(m_hID, hProgWnd, lpTitleText);
}
bool CExtAPI::progbox_SetProgress(ZVAR hProgWnd, __int64 p, __int64 pmax) {
	if(!m_lpfn_progbox_SetProgress) {
		ReportError("CExtAPI::progbox_SetProgress error - function not found");
		return false;
	}

	return m_lpfn_progbox_SetProgress(m_hID, hProgWnd, p, pmax);
}

//
// combo box functions
//

bool CExtAPI::combosel_SetOptions(ZVAR hComboSel, LPCSTR lpOptionsStr) {
	if(!m_lpfn_combosel_SetOptions) {
		ReportError("CExtAPI::combosel_SetOptions error - function not found");
		return false;
	}

	return m_lpfn_combosel_SetOptions(m_hID, hComboSel, lpOptionsStr);
}
LPCSTR CExtAPI::combosel_GetOptions(ZVAR hComboSel) {
	if(!m_lpfn_combosel_GetOptions) {
		ReportError("CExtAPI::combosel_GetOptions error - function not found");
		return false;
	}

	return m_lpfn_combosel_GetOptions(m_hID, hComboSel);
}
bool CExtAPI::combosel_SetCurSel(ZVAR hComboSel, LPCSTR lpCurSelStr) {
	if(!m_lpfn_combosel_SetCurSel) {
		ReportError("CExtAPI::combosel_SetCurSel error - function not found");
		return false;
	}

	return m_lpfn_combosel_SetCurSel(m_hID, hComboSel, lpCurSelStr);
}
LPCSTR CExtAPI::combosel_GetCurSel(ZVAR hComboSel) {
	if(!m_lpfn_combosel_GetCurSel) {
		ReportError("CExtAPI::combosel_GetCurSel error - function not found");
		return false;
	}

	return m_lpfn_combosel_GetCurSel(m_hID, hComboSel);
}

//
// project functions
//

bool CExtAPI::project_GetHeightfieldRange(float& minval, float& maxval) {
	if(!m_lpfn_project_GetHeightfieldRange) {
		ReportError("CExtAPI::project_GetHeightfieldRange error - function not found");
		return false;
	}

	return m_lpfn_project_GetHeightfieldRange(m_hID, minval, maxval);
}