#pragma once
#include "afx.h"
#include "extapi_defines.h"

// app function handle defs

typedef FARPROC (__stdcall* LPFNAPI_app_GetFuncHandle)(LPVOID hID, LPCSTR lpFuncHandle);
typedef void (__stdcall* LPFNAPI_app_ReportError)(LPVOID hID, LPCSTR lpErrorStr);

// list function handle defs

typedef ZLIST (__stdcall* LPFNAPI_list_GetList)(LPVOID hID, LPCSTR lpListName);
typedef long (__stdcall* LPFNAPI_list_GetSize)(LPVOID hID, ZLIST hList);
typedef ZLIST (__stdcall* LPFNAPI_list_CreateMember)(LPVOID hID, ZLIST hList, long VarID, LPCSTR lpVarName);
typedef ZLIST (__stdcall* LPFNAPI_list_GetMemberByIndex)(LPVOID hID, ZLIST hList, long index);
typedef ZLIST (__stdcall* LPFNAPI_list_GetMemberByName)(LPVOID hID, ZLIST hList, LPCSTR lpVarName);
typedef bool (__stdcall* LPFNAPI_list_DeleteMemberByIndex)(LPVOID hID, ZLIST hList, long index);
typedef bool (__stdcall* LPFNAPI_list_DeleteMemberByName)(LPVOID hID, ZLIST hList, LPCSTR lpVarName);
typedef void (__stdcall* LPFNAPI_list_DeleteMembers)(LPVOID hID, ZLIST hList);

// map function handle defs

typedef ZMAP (__stdcall* LPFNAPI_map_GetMap)(LPVOID hID, LPCSTR lpMapName);
typedef bool (__stdcall* LPFNAPI_map_Init)(LPVOID hID, ZMAP hMap, long nx, long ny, long MapTypeID, float HorizScale, bool WrapFlag);
typedef bool (__stdcall* LPFNAPI_map_Free)(LPVOID hID, ZMAP hMap);
typedef long (__stdcall* LPFNAPI_map_GetPixelSize)(LPVOID hID, ZMAP hMap);
typedef bool (__stdcall* LPFNAPI_map_GetWrapFlag)(LPVOID hID, ZMAP hMap);
typedef long (__stdcall* LPFNAPI_map_GetMapType)(LPVOID hID, ZMAP hMap);
typedef long (__stdcall* LPFNAPI_map_GetWidth)(LPVOID hID, ZMAP hMap);
typedef long (__stdcall* LPFNAPI_map_GetHeight)(LPVOID hID, ZMAP hMap);
typedef bool (__stdcall* LPFNAPI_map_GetPixel)(LPVOID hID, ZMAP hMap, long x, long y, void* pValue);
typedef bool (__stdcall* LPFNAPI_map_SetPixel)(LPVOID hID, ZMAP hMap, long x, long y, void* pValue);
typedef bool (__stdcall* LPFNAPI_map_GetMinMaxAlt)(LPVOID hID, ZMAP hMap, float& minval, float& maxval);
typedef float (__stdcall* LPFNAPI_map_GetHorizScale)(LPVOID hID, ZMAP hMap);
typedef bool (__stdcall* LPFNAPI_map_GetMosaicFlag)(LPVOID hID, ZMAP hMap);
typedef long (__stdcall* LPFNAPI_map_GetMosaicTileSize)(LPVOID hID, ZMAP hMap);
typedef long (__stdcall* LPFNAPI_map_GetMosaicTilesX)(LPVOID hID, ZMAP hMap);
typedef long (__stdcall* LPFNAPI_map_GetMosaicTilesY)(LPVOID hID, ZMAP hMap);

// format function handle defs

typedef ZFORMAT (__stdcall* LPFNAPI_format_Create)(LPVOID hID, LPCTSTR lpMapName, LPCSTR lpLongName, LPCSTR lpFileExt, LPCSTR lpNativeProgramName);
typedef bool (__stdcall* LPFNAPI_format_SetFlags)(LPVOID hID, ZFORMAT hFormat, bool Native, bool Mosaic);
typedef LPCSTR (__stdcall* LPFNAPI_format_GetExt)(LPVOID hID, ZFORMAT hFormat);
typedef ZLIST (__stdcall* LPFNAPI_format_GetOptionList)(LPVOID hID, ZFORMAT hFormat);
typedef bool (__stdcall* LPFNAPI_format_GetOptionValue)(LPVOID hID, ZFORMAT hFormat, LPCSTR lpOptionName, long VarID, void* pValue);
typedef bool (__stdcall* LPFNAPI_format_SetOptionValue)(LPVOID hID, ZFORMAT hFormat, LPCSTR lpOptionName, long VarID, void* pValue);

// prog box function handle defs

typedef bool (__stdcall* LPFNAPI_progbox_ShowWnd)(LPVOID hID, ZVAR hProgWnd);
typedef bool (__stdcall* LPFNAPI_progbox_HideWnd)(LPVOID hID, ZVAR hProgWnd);
typedef bool (__stdcall* LPFNAPI_progbox_SetTitle)(LPVOID hID, ZVAR hProgWnd, LPCSTR lpTitleText);
typedef bool (__stdcall* LPFNAPI_progbox_SetProgress)(LPVOID hID, ZVAR hProgWnd, __int64 p, __int64 pmax);

// combo box function handle defs

typedef bool (__stdcall* LPFNAPI_combosel_SetOptions)(LPVOID hID, ZVAR hComboSel, LPCSTR lpOptionsStr);
typedef LPCSTR (__stdcall* LPFNAPI_combosel_GetOptions)(LPVOID hID, ZVAR hComboSel);
typedef bool (__stdcall* LPFNAPI_combosel_SetCurSel)(LPVOID hID, ZVAR hComboSel, LPCSTR lpCurSelStr);
typedef LPCSTR (__stdcall* LPFNAPI_combosel_GetCurSel)(LPVOID hID, ZVAR hComboSel);

// project function handle defs

typedef bool (__stdcall* LPFNAPI_project_GetHeightfieldRange)(LPVOID hID, float& minval, float& maxval);


class CExtAPI :
	public CObject
{
public:
	CExtAPI(void);
	~CExtAPI(void);

	bool InitAPI(FARPROC pFunc, LPVOID hID);

	// app functions

	FARPROC GetFuncHandle(LPCTSTR lpFuncName);
	void	ReportError(LPCSTR lpErrorMessage);

	// list functions

	ZLIST	list_GetList(LPCSTR lpListName);
	long	list_GetSize(ZLIST hList);
	ZLIST	list_CreateMember(ZLIST hList, long VarID, LPCSTR lpVarName);
	ZLIST	list_GetMemberByIndex(ZLIST hList, long index);
	ZLIST	list_GetMemberByName(ZLIST hList, LPCSTR lpVarName);
	bool	list_DeleteMemberByIndex(ZLIST hList, long index);
	bool	list_DeleteMemberByName(ZLIST hList, LPCSTR lpVarName);
	void	list_DeleteMembers(ZLIST hList);

	// map functions
	ZMAP	map_GetMap(LPCSTR lpMapName);
	bool	map_Init(ZMAP hMap, long nx, long ny, long MapTypeID, float HorizScale, bool WrapFlag);
	bool	map_Free(ZMAP hMap);
	long	map_GetMapType(ZMAP hMap);
	long	map_GetPixelSize(ZMAP hMap);
	bool	map_GetWrapFlag(ZMAP hMap);
	long	map_GetWidth(ZMAP hMap);
	long	map_GetHeight(ZMAP hMap);
	bool	map_GetPixel(ZMAP hMap, long x, long y, void* pValue);
	bool	map_SetPixel(ZMAP hMap, long x, long y, void* pValue);
	bool	map_GetMinMaxAlt(ZMAP hMap, float& minval, float& maxval);
	float	map_GetHorizScale(ZMAP hMap);
	bool	map_GetMosaicFlag(ZMAP hMap);
	long	map_GetMosaicTileSize(ZMAP hMap);
	long	map_GetMosaicTilesX(ZMAP hMap);
	long	map_GetMosaicTilesY(ZMAP hMap);

	//format functions

	ZFORMAT format_Create(LPCTSTR lpMapName, LPCSTR lpLongName, LPCSTR lpFileExt, LPCSTR lpNativeProgramName);
	bool	format_SetFlags(ZFORMAT hFormat, bool Native, bool Mosaic);
	LPCSTR	format_GetExt(ZFORMAT hFormat);
	ZLIST	format_GetOptionList(ZFORMAT hFormat);
	bool	format_GetOptionValue(ZFORMAT hFormat, LPCSTR lpOptionName, long VarID, void* pValue);
	bool	format_SetOptionValue(ZFORMAT hFormat, LPCSTR lpOptionName, long VarID, void* pValue);

	// prog box functions

	bool progbox_ShowWnd(ZVAR hProgWnd);
	bool progbox_HideWnd(ZVAR hProgWnd);
	bool progbox_SetTitle(ZVAR hProgWnd, LPCSTR lpTitleText);
	bool progbox_SetProgress(ZVAR hProgWnd, __int64 p, __int64 pmax);

	// combo box functions

	bool	combosel_SetOptions(ZVAR hComboSel, LPCSTR lpOptionsStr);
	LPCSTR	combosel_GetOptions(ZVAR hComboSel);
	bool	combosel_SetCurSel(ZVAR hComboSel, LPCSTR lpCurSelStr);
	LPCSTR	combosel_GetCurSel(ZVAR hComboSel);

	// project functions

	bool	project_GetHeightfieldRange(float& minval, float& maxval);

protected:

	// session ID's

	LPVOID m_hID;

	// app function handles

	LPFNAPI_app_GetFuncHandle	m_lpfnGetFuncHandle;
	LPFNAPI_app_ReportError		m_lpfnReportError;

	// list function handles

	LPFNAPI_list_GetList				m_lpfn_list_GetList;
	LPFNAPI_list_GetSize				m_lpfn_list_GetSize;
	LPFNAPI_list_CreateMember			m_lpfn_list_CreateMember;
	LPFNAPI_list_GetMemberByIndex		m_lpfn_list_GetMemberByIndex;
	LPFNAPI_list_GetMemberByName		m_lpfn_list_GetMemberByName;
	LPFNAPI_list_DeleteMemberByIndex	m_lpfn_list_DeleteMemberByIndex;
	LPFNAPI_list_DeleteMemberByName		m_lpfn_list_DeleteMemberByName;
	LPFNAPI_list_DeleteMembers			m_lpfn_list_DeleteMembers;

	// map function handles

	LPFNAPI_map_GetMap			m_lpfn_map_GetMap;
	LPFNAPI_map_Init			m_lpfn_map_Init;
	LPFNAPI_map_Free			m_lpfn_map_Free;
	LPFNAPI_map_GetMapType		m_lpfn_map_GetMapType;
	LPFNAPI_map_GetPixelSize	m_lpfn_map_GetPixelSize;
	LPFNAPI_map_GetWrapFlag		m_lpfn_map_GetWrapFlag;
	LPFNAPI_map_GetWidth		m_lpfn_map_GetWidth;
	LPFNAPI_map_GetHeight		m_lpfn_map_GetHeight;
	LPFNAPI_map_GetPixel		m_lpfn_map_GetPixel;
	LPFNAPI_map_SetPixel		m_lpfn_map_SetPixel;
	LPFNAPI_map_GetMinMaxAlt	m_lpfn_map_GetMinMaxAlt;
	LPFNAPI_map_GetHorizScale	m_lpfn_map_GetHorizScale;
	LPFNAPI_map_GetMosaicFlag		m_lpfn_map_GetMosaicFlag;
	LPFNAPI_map_GetMosaicTileSize	m_lpfn_map_GetMosaicTileSize;
	LPFNAPI_map_GetMosaicTilesX		m_lpfn_map_GetMosaicTilesX;
	LPFNAPI_map_GetMosaicTilesY		m_lpfn_map_GetMosaicTilesY;

	// format function handles

	LPFNAPI_format_Create			m_lpfn_format_Create;
	LPFNAPI_format_SetFlags			m_lpfn_format_SetFlags;
	LPFNAPI_format_GetExt			m_lpfn_format_GetExt;
	LPFNAPI_format_GetOptionList	m_lpfn_format_GetOptionList;
	LPFNAPI_format_GetOptionValue	m_lpfn_format_GetOptionValue;
	LPFNAPI_format_SetOptionValue	m_lpfn_format_SetOptionValue;

	// prog box function handles

	LPFNAPI_progbox_ShowWnd			m_lpfn_progbox_ShowWnd;
	LPFNAPI_progbox_HideWnd			m_lpfn_progbox_HideWnd;
	LPFNAPI_progbox_SetTitle		m_lpfn_progbox_SetTitle;
	LPFNAPI_progbox_SetProgress		m_lpfn_progbox_SetProgress;

	// combo box function handles

	LPFNAPI_combosel_SetOptions		m_lpfn_combosel_SetOptions;
	LPFNAPI_combosel_GetOptions		m_lpfn_combosel_GetOptions;
	LPFNAPI_combosel_SetCurSel		m_lpfn_combosel_SetCurSel;
	LPFNAPI_combosel_GetCurSel		m_lpfn_combosel_GetCurSel;

	// project function handle defs

	LPFNAPI_project_GetHeightfieldRange		m_lpfn_project_GetHeightfieldRange;
};
